@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.!
 
+#include "unigram_model.h"
+
 #include <cmath>
 #include <map>
 #include <string>
@@ -22,7 +24,6 @@
 #include "testharness.h"
 #include "third_party/absl/strings/str_cat.h"
 #include "third_party/absl/strings/str_join.h"
-#include "unigram_model.h"
 #include "util.h"
 
 namespace sentencepiece {
@@ -161,11 +162,11 @@ TEST(LatticeTest, InsertTest) {
 TEST(LatticeTest, ViterbiFromIncompleteLatticeTest) {
   Lattice lattice;
   lattice.SetSentence("ABC");
-  EXPECT_TRUE(lattice.Viterbi().empty());
+  EXPECT_TRUE(lattice.Viterbi().first.empty());
 
   // Still incomplete
   lattice.Insert(0, 1);
-  EXPECT_TRUE(lattice.Viterbi().empty());
+  EXPECT_TRUE(lattice.Viterbi().first.empty());
 
   lattice.Insert(1, 1);
   lattice.Insert(2, 1);
@@ -198,16 +199,16 @@ TEST(LatticeTest, ViterbiTest) {
   InsertWithScore(&lattice, 0, 1, 0.0);  // A
   InsertWithScore(&lattice, 1, 1, 0.0);  // B
   InsertWithScore(&lattice, 2, 1, 0.0);  // C
-  EXPECT_EQ("A B C", GetTokenized(lattice.Viterbi()));
+  EXPECT_EQ("A B C", GetTokenized(lattice.Viterbi().first));
 
   InsertWithScore(&lattice, 0, 2, 2.0);  // AB
-  EXPECT_EQ("AB C", GetTokenized(lattice.Viterbi()));
+  EXPECT_EQ("AB C", GetTokenized(lattice.Viterbi().first));
 
   InsertWithScore(&lattice, 1, 2, 5.0);  // BC
-  EXPECT_EQ("A BC", GetTokenized(lattice.Viterbi()));
+  EXPECT_EQ("A BC", GetTokenized(lattice.Viterbi().first));
 
   InsertWithScore(&lattice, 0, 3, 10.0);  // ABC
-  EXPECT_EQ("ABC", GetTokenized(lattice.Viterbi()));
+  EXPECT_EQ("ABC", GetTokenized(lattice.Viterbi().first));
 }
 
 TEST(LatticeTest, NBestTest) {
@@ -221,21 +222,174 @@ TEST(LatticeTest, NBestTest) {
   InsertWithScore(&lattice, 1, 2, 5.0);   // BC
   InsertWithScore(&lattice, 0, 3, 10.0);  // ABC
 
-  auto nbests = lattice.NBest(10);
+  auto nbests = lattice.NBest(10, false, 0.0);
   EXPECT_EQ(4, nbests.size());
 
-  EXPECT_EQ("ABC", GetTokenized(nbests[0]));
-  EXPECT_EQ("A BC", GetTokenized(nbests[1]));
-  EXPECT_EQ("AB C", GetTokenized(nbests[2]));
-  EXPECT_EQ("A B C", GetTokenized(nbests[3]));
+  EXPECT_EQ("ABC", GetTokenized(nbests[0].first));
+  EXPECT_EQ("A BC", GetTokenized(nbests[1].first));
+  EXPECT_EQ("AB C", GetTokenized(nbests[2].first));
+  EXPECT_EQ("A B C", GetTokenized(nbests[3].first));
 
-  auto nbests0 = lattice.NBest(0);
+  auto nbests0 = lattice.NBest(0, false, 0.0);
   EXPECT_TRUE(nbests0.empty());
 
-  auto nbests1 = lattice.NBest(1);
+  auto nbests1 = lattice.NBest(1, false, 0.0);
   EXPECT_EQ(nbests1.size(), 1);
 }
 
+TEST(LatticeTest, NBestSampleTest) {
+  Lattice lattice;
+  lattice.SetSentence("ABC");
+
+  InsertWithScore(&lattice, 0, 1, 0.0);  // A
+  InsertWithScore(&lattice, 1, 1, 0.0);  // B
+  InsertWithScore(&lattice, 2, 1, 0.1);  // C
+  InsertWithScore(&lattice, 0, 2, 0.2);  // AB
+  InsertWithScore(&lattice, 1, 2, 0.5);  // BC
+  InsertWithScore(&lattice, 0, 3, 1.0);  // ABC
+
+  // Calculate expected probabilities of each path
+  // Note that sampling without replacement affects the expected frequencies!
+  const std::vector<double> kTheta = {0.0, 0.01, 0.5, 0.7, 1.0};
+  for (const auto theta : kTheta) {
+    std::vector<std::string> strings = {"ABC", "AB C", "A BC", "A B C"};
+    std::map<std::string, float> probs;
+    probs["ABC"] = std::exp(theta * 1.0);
+    probs["AB C"] = std::exp(theta * (0.2 + 0.1));
+    probs["A BC"] = std::exp(theta * (0.0 + 0.5));
+    probs["A B C"] = std::exp(theta * (0.0 + 0.0 + 0.1));
+
+    for (const auto &it : strings) {
+      EXPECT_EQ(1, probs.count(it));
+    }
+
+    double Z = 0.0;
+    for (const auto &it : probs) Z += it.second;
+    for (auto &it : probs) it.second /= Z;
+
+    std::map<std::pair<std::string, std::string>, float> pair_probs;
+    for (const auto first : strings) {
+      for (const auto second : strings) {
+        if (first == second) {
+          pair_probs[std::make_pair(first, second)] = 0;
+        } else {
+          float first_prob = probs[first];
+          float second_prob = probs[second] / (1 - first_prob);
+          pair_probs[std::make_pair(first, second)] = first_prob * second_prob;
+        }
+      }
+    }
+
+    std::map<std::string, float> inclusion_probs;
+    for (const auto string : strings) {
+      float inclusion_prob = 0.0;
+      for (const auto other_string : strings) {
+        inclusion_prob += pair_probs[std::make_pair(string, other_string)];
+      }
+      for (const auto other_string : strings) {
+        inclusion_prob += pair_probs[std::make_pair(other_string, string)];
+      }
+      inclusion_probs[string] = inclusion_prob / 2;
+    }
+
+    int kTrials = 10000;
+
+    std::vector<int> kNumSamples = {1, 2};
+
+    for (const auto num_samples : kNumSamples) {
+      std::map<std::string, int> counts;
+      for (int i = 0; i < kTrials; i++) {
+        auto nbests = lattice.NBest(num_samples, true, theta);
+        for (const auto nbest : nbests) {
+          counts[GetTokenized(nbest.first)]++;
+        }
+      }
+
+      EXPECT_EQ(inclusion_probs.size(), counts.size());
+      // If we take multiple samples WOR, we have to use corrected probs.
+      std::map<std::string, float> probs_to_use =
+          (num_samples == 1 ? probs : inclusion_probs);
+
+      for (const auto &it : probs_to_use) {
+        EXPECT_NEAR(it.second, 1.0 * counts[it.first] / (kTrials * num_samples),
+                    0.02);
+      }
+    }
+  }
+}
+
+TEST(LatticeTest, CalculateEntropyTest) {
+  Lattice lattice;
+  lattice.SetSentence("ABC");
+
+  InsertWithScore(&lattice, 0, 1, 0.0);  // A
+  InsertWithScore(&lattice, 1, 1, 0.0);  // B
+  InsertWithScore(&lattice, 2, 1, 0.1);  // C
+  InsertWithScore(&lattice, 0, 2, 0.2);  // AB
+  InsertWithScore(&lattice, 1, 2, 0.5);  // BC
+  InsertWithScore(&lattice, 0, 3, 1.0);  // ABC
+
+  // Calculate expected probabilities of each path
+  const std::vector<double> kTheta = {0.0, 0.01, 0.5, 0.7, 1.0};
+  for (const auto theta : kTheta) {
+    std::vector<std::string> strings = {"ABC", "AB C", "A BC", "A B C"};
+    std::map<std::string, float> probs;
+    probs["ABC"] = std::exp(theta * 1.0);
+    probs["AB C"] = std::exp(theta * (0.2 + 0.1));
+    probs["A BC"] = std::exp(theta * (0.0 + 0.5));
+    probs["A B C"] = std::exp(theta * (0.0 + 0.0 + 0.1));
+
+    double Z = 0.0;
+    for (const auto &it : probs) Z += it.second;
+    for (auto &it : probs) it.second /= Z;
+
+    for (const auto &it : strings) {
+      EXPECT_EQ(1, probs.count(it));
+    }
+    float entropy = 0.0;
+    for (const auto &it : probs) {
+      entropy += (it.second * std::log(it.second));
+    }
+    EXPECT_NEAR(-entropy, lattice.CalculateEntropy(theta), 0.02);
+  }
+}
+
+TEST(LatticeTest, ForwardAlgorithmTest) {
+  Lattice lattice;
+  lattice.SetSentence("ABC");
+
+  InsertWithScore(&lattice, 0, 1, 0.0);  // A
+  InsertWithScore(&lattice, 1, 1, 0.0);  // B
+  InsertWithScore(&lattice, 2, 1, 0.1);  // C
+  InsertWithScore(&lattice, 0, 2, 0.2);  // AB
+  InsertWithScore(&lattice, 1, 2, 0.5);  // BC
+  InsertWithScore(&lattice, 0, 3, 1.0);  // ABC
+
+  const std::vector<float> kTheta = {0.0, 0.01, 0.5, 0.7, 1.0};
+  for (const auto theta : kTheta) {
+    std::vector<float> alpha = lattice.ForwardAlgorithm(theta);
+    EXPECT_EQ(alpha.size(), 8);  // 6 nodes, plus BOS, EOS
+    // only alpha[C], alpha[EOS] have non-zero alpha
+    for (int i : {0, 1, 2, 3}) {
+      for (const auto &node : lattice.begin_nodes(i)) {
+        if (i < 2) {
+          EXPECT_EQ(alpha[node->node_id], 0.0);
+        } else if (i == 2) {
+          float Z =
+              std::log(std::exp(theta * (0.0 + 0.0)) + std::exp(theta * 0.2));
+          EXPECT_EQ(alpha[node->node_id], Z);
+        } else if (i == 3) {
+          float Z = std::log(std::exp(theta * (0.0 + 0.0 + 0.1)) +  // A + B + C
+                             std::exp(theta * (0.2 + 0.1)) +        // AB + C
+                             std::exp(theta * (0.0 + 0.5)) +        // A + BC
+                             std::exp(theta * 1.0));                // ABC
+          EXPECT_EQ(Z, alpha[node->node_id]);
+        }
+      }
+    }
+  }
+}
+
 TEST(LatticeTest, PopulateMarginalTest) {
   Lattice lattice;
   lattice.SetSentence("ABC");
@@ -361,6 +515,102 @@ TEST(UnigramModelTest, SetUnigramModelTest) {
             model.model_proto().SerializeAsString());
 }
 
+TEST(UnigramModelTest, SampleEncodeAndScoreTest) {
+  // Test whether inclusion probabilities are correct
+  ModelProto model_proto = MakeBaseModelProto();
+  AddPiece(&model_proto, "A", 0.0);    // 3
+  AddPiece(&model_proto, "B", 0.0);    // 4
+  AddPiece(&model_proto, "C", 0.1);    // 5
+  AddPiece(&model_proto, "AB", 0.2);   // 6
+  AddPiece(&model_proto, "BC", 0.5);   // 7
+  AddPiece(&model_proto, "ABC", 1.0);  // 8
+
+  Model model(model_proto);
+
+  Lattice lattice;
+  lattice.SetSentence("ABC");
+  model.PopulateNodes(&lattice);
+
+  std::vector<float> kTheta = {0.0, 1.0};
+
+  for (const auto theta : kTheta) {
+    std::vector<std::string> strings = {"ABC", "AB C", "A BC", "A B C"};
+    std::map<std::string, float> probs;
+    probs["ABC"] = std::exp(theta * 1.0);
+    probs["AB C"] = std::exp(theta * (0.2 + 0.1));
+    probs["A BC"] = std::exp(theta * (0.0 + 0.5));
+    probs["A B C"] = std::exp(theta * (0.0 + 0.0 + 0.1));
+
+    for (const auto &it : strings) {
+      EXPECT_EQ(1, probs.count(it));
+    }
+
+    double Z = 0.0;
+    for (const auto &it : probs) Z += it.second;
+    for (auto &it : probs) it.second /= Z;
+
+    std::map<std::pair<std::string, std::string>, float> pair_probs;
+    for (const auto first : strings) {
+      for (const auto second : strings) {
+        if (first == second) {
+          pair_probs[std::make_pair(first, second)] = 0;
+        } else {
+          float first_prob = probs[first];
+          float second_prob = probs[second] / (1 - first_prob);
+          pair_probs[std::make_pair(first, second)] = first_prob * second_prob;
+        }
+      }
+    }
+
+    std::map<std::string, float> inclusion_probs;
+    for (const auto string : strings) {
+      float inclusion_prob = 0.0;
+      for (const auto other_string : strings) {
+        inclusion_prob += pair_probs[std::make_pair(string, other_string)];
+      }
+      for (const auto other_string : strings) {
+        inclusion_prob += pair_probs[std::make_pair(other_string, string)];
+      }
+      inclusion_probs[string] = inclusion_prob / 2;
+    }
+    std::vector<int> kNumSamples = {1, 2};
+
+    for (const auto num_samples : kNumSamples) {
+      std::map<std::string, int> counts;
+      std::map<std::string, float> scores;
+      int kTrials = 50000;
+      for (int i = 0; i < kTrials; i++) {
+        NBestEncodeResult sample =
+            model.SampleEncodeAndScore("ABC", theta, num_samples, true, false);
+
+        for (const auto &it : sample) {
+          std::vector<std::string> tokens;
+          for (const auto &inner_it : it.first) {
+            tokens.push_back(std::string(inner_it.first));
+          }
+          std::string sample_string = absl::StrJoin(tokens, " ");
+          counts[sample_string] += 1;
+          // use the fact that E(1_{i in sample} / score of i) = 1
+          // see https://arxiv.org/pdf/1903.06059.pdf appendix D
+          scores[sample_string] += std::exp(-it.second);
+        }
+      }
+
+      // Check that counts and probs are correct
+      std::map<std::string, float> probs_to_use =
+          (num_samples == 1 ? probs : inclusion_probs);
+
+      for (const auto &it : scores) Z += it.second;
+      for (const auto &it : probs_to_use) {
+        EXPECT_NEAR(it.second, 1.0 * counts[it.first] / (kTrials * num_samples),
+                    0.02);
+        // The expectation is quite loose, use a higher tolerance
+        EXPECT_NEAR(1.0, scores[it.first] / kTrials, 0.20);
+      }
+    }
+  }
+}
+
 TEST_P(UnigramModelTest, PieceToIdTest) {
   ModelProto model_proto = MakeBaseModelProto();
 
