@@ -36,9 +36,12 @@
 #include <pthread.h>
 #endif
 
-#if !defined(__APPLE__) && !defined(_WIN32) && BYTE_ORDER == __BIG_ENDIAN
+#if !defined(__APPLE__) && !defined(_WIN32)
+#include <endian.h>
+#if BYTE_ORDER == __BIG_ENDIAN
 #define IS_BIG_ENDIAN
 #endif
+#endif
 
 namespace sentencepiece {
 template <typename T>
